#include <linux/ps2/ee.h>
#include <ps2dma.h>
#include <ps2vpu.h>
#include <stdio.h>
#include <unistd.h>
#include <libvu0.h>
#include <ps2vpufile.h>
#include <sys/mman.h>
#include <errno.h>

IMPORT_VPU_SYMBOL(__u128 *, My_dma_start89, 0)
IMPORT_VPU_SYMBOL(__u128 *, vu0_waste_start, 0)
IMPORT_VPU_SYMBOL(__u128 *, vu1_waste_start, 0)

int main() {

        ps2_vpu *vdev0, *vdev1;
        VPUFILE *vfd1, *vfd2;

	float *q , *r;
	int i=0,j=0;
	float ratio;
	int result;

	/* Prepare access to VU0 */
        vdev0 = ps2_vpu_open(0);
        vdev1 = ps2_vpu_open(1);
        ps2_vpu_reset(vdev0);
        ps2_vpu_reset(vdev1);

	/* Retrieve mmaped shared mem of VU Mem0 */
	q = ps2_vpu_data(vdev0);	
	r = ps2_vpu_data(vdev1);	

	/* Set 2 vectors. */
	q[0] = 1.0; q[1] = 1.0; q[2] = 1.0; q[3] = 1.0; 
	q[4] = 2.0; q[5] = 3.0; q[6] = 4.0; q[7] = 5.0;
	r[0] = 1.0; r[1] = 1.0; r[2] = 1.0; r[3] = 1.0; 
	r[4] = 2.0; r[5] = 3.0; r[6] = 4.0; r[7] = 5.0;

	/* Prepare our vector addition code to be sent to VUs */
        vfd1 = vpuobj_open("vu0waste.elf", O_DATA_PS2MEM);
        if (vfd1 < 0) {
                printf("couldn't write adder.elf to vpu0\n");
                perror("vpu_open");
                exit(1);
        }
        vfd2 = vpuobj_open("vu1waste.elf", O_DATA_PS2MEM);
        if (vfd2 < 0) {
                printf("couldn't write adder.elf to vpu1\n");
                perror("vpu_open");
                exit(1);
        }
	
	/* Actually send the code to VU0 and VU1*/
	ps2_dma_send(ps2_vpu_fd(vdev0), vfd1, (ps2_dmatag *)vu0_waste_start);
	ps2_dma_send(ps2_vpu_fd(vdev1), vfd2, (ps2_dmatag *)vu1_waste_start);
	
	printf("\n\n\n\n\n\n\n\n\n\n\n");
	printf("Test to show that the technique for starting/running the VUs described\nin http://pakl.net/ps2vufaq.html does indeed allow them to\nrun in parallel.\n\n");
	printf("First Test\n--------------\n  Run and time each vu separately.\n");
	ps2_vpu_start(vdev1,0);
	while (ps2_vpu_busy(vdev1)) { j++; }
	ps2_vpu_start(vdev0,0);
	while (ps2_vpu_busy(vdev0)) { i++; }
	
	printf("Results:\n----------\n This C program was able to count to %d while waiting for VU0, and to %d while waiting for VU1.\n", i, j);
	ratio = (float) j / (float) i;
	printf("The ratio vu1/vu0 was: %f (should be close to 2x because of the code)\n", ratio);

	printf("\n\nSecond Test\n-------------\n Start VU1 (2x longer) then attempt to start VU0 as soon as possible.  
 If ratio is smaller than 2x, vu0 and vu1 ran simultaneously.\n");

	ps2_vpu_start(vdev1,0);   /* Exec code that's 2x as long as vu0's */
	ps2_vpu_start(vdev0,0);
	while ( ps2_vpu_busy(vdev0)) { i++; }
	while ( ps2_vpu_busy(vdev1)) { j++; }

	/*printf("vu0 =  : %f %f %f %f \n", q[0], q[1], q[2], q[3]); 
	printf("vu1 =  : %f %f %f %f \n", r[0], r[1], r[2], r[3]); */

	printf("Results:\n----------\n This C program was able to start vu1 and then vu0,\ncount to %d while first waiting for VU0,\nand then had to count to %d while waiting for VU1.\n", i, j);
	ratio = (float) j / (float) i;
	printf("The ratio vu1/vu0 was: %f (should be signif. less than 2.0)\n", ratio);

        ps2_vpu_close(vdev1);
        ps2_vpu_close(vdev0);

}
